"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
// Main process manager
const manager = new utils_1.ProcessManager();
/**
 * Launch a shell command, return a Promise
 * @param command a shell command to launch
 */
function shell(command, name = null) {
    return manager.launchProcess(name, command);
}
exports.shell = shell;
/**
 * Run a command and return a Promise
 * @param command
 * string: shell command
 * Array<Command>: run the array of commands sequentially
 * { name: Command, ... }: run the dict of commands in parallel
 */
function run(command, name = null) {
    if (typeof (command) == "string") {
        return shell(command, name);
    }
    else if (command instanceof Function) {
        return command();
    }
    else if (command instanceof Array) {
        return sequence(command);
    }
    else {
        return Promise.all(Object.keys(command).map(key => run(command[key], key))).then(() => { });
    }
}
exports.run = run;
/**
 * Run a list of commands sequentially
 * @param commands commands
 */
function sequence(commands) {
    return __awaiter(this, void 0, void 0, function* () {
        for (let cmd of commands) {
            yield run(cmd);
        }
    });
}
exports.sequence = sequence;
